#ifndef __LOCAL_HEAP_H__
#define __LOCAL_HEAP_H__

#include <cstdlib>
#include "memlib.h"
#include "allocator_util.h"
#include "GlobalHeap.h"

/* SUPERBLOCK STRUCT IN LOCAL HEAP:
 * 
 * SIZE 
 * NEXT 
 * OWNER
 * USAGE [% of superblock used]
 * BITMASK 
 * data
 *
 * localHeap only stores non-full superblock
 */

class LocalHeap
{
 public:
  LocalHeap();
  ~LocalHeap();
  int init(GlobalHeap *globalHeap, int heapId);

  void *malloc(size_t size);
  void free(void *block);

  int check();
 private:
  void *requestNewSuperBlock(int blockIndex);
  void *findEmptyBlock(int blockIndex);
  void _freeBlock(void *superblock, void *ptr, int blockIndex);
  bool isSuperBlockEmpty(void *superblock, int blockIndex);

  void *_markBlock(void *superblock, int kbyte, int blockIndex, int maxm = 32);
  void _addToList(void *superblock, int blockIndex);
  void _removeFromList(void *superblock, int blockIndex);
  void _clearBitmask(void *superblock, int blockIndex);

  void *buckets[MAX_BLOCK_POW + 1];
  pthread_mutex_t localMutex;
  GlobalHeap *globalHeap;
  int heapId;
};

#endif //__LOCAL_HEAP_H__
