/* allocator_util.h
 * provides important helper functions used by 
 * allocator,  LocalHeap and GlobalHeap
 */
#ifndef __ALLOCATOR_UTIL_H__
#define __ALLOCATOR_UTIL_H__

#include <cstdio>
#include <assert.h>

#define PAGE_SIZE 4096
#define SUPERBLOCK_SIZE PAGE_SIZE
#define N_LOCAL_HEAPS 16
#define BIG_BLOCK_HEADER (sizeof(size_t))

#define LOCAL_SIZE_START 0
#define LOCAL_NEXT_START (sizeof(size_t))
#define LOCAL_OWNER_START (sizeof(size_t) * 2)
#define LOCAL_USAGE_START (sizeof(size_t) * 3)
#define LOCAL_BITMASK_START (sizeof(size_t) * 4)
#define BITMASK_SIZE (SUPERBLOCK_SIZE / 8)
#define LOCAL_DATA_START (LOCAL_BITMASK_START + BITMASK_SIZE)

#define MAX_BLOCK_POW 10


int ROUNDUP(int a, int b);

int getBlockIndex(size_t size);
bool isBigRequest(size_t size);

void *getSuperBlockFromPointer(void *ptr, void *mem_lo);
int getBitmaskSize(int blockIndex);

void setBlockSize(void *ptr, size_t size);
size_t getBlockSize(void *ptr);
void* getBlockNext(void *ptr);
void setBlockNext(void *ptr, void *val);
size_t getBlockOwner(void *ptr);
void setBlockOwner(void *ptr, size_t owner);
size_t getBlockUsage(void *ptr);
void setBlockUsage(void *ptr, size_t usage);
void *getBlockBitmask(void *ptr);
void *getBlockData(void *ptr);

int doBitCount(unsigned int val);
int findIndexOfFirstZeroBit(int val);


#endif //__ALLOCATOR_UTIL_H__
